//
// RequestProcessor.h
//
// $Id: //poco/Main/WebWidgets/include/Poco/WebWidgets/RequestProcessor.h#3 $
//
// Library: WebWidgets
// Package: Core
// Module:  RequestProcessor
//
// Definition of the RequestProcessor class.
//
// Copyright (c) 2008, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#ifndef WebWidgets_RequestProcessor_INCLUDED
#define WebWidgets_RequestProcessor_INCLUDED


#include "Poco/WebWidgets/WebWidgets.h"
#include <ostream>


namespace Poco {
namespace Net {
	class NameValueCollection;
	class HTTPServerResponse;
} }


namespace Poco {
namespace WebWidgets {


class WebWidgets_API RequestProcessor
	/// This interface class defines the callback methods for
	/// Cells or Views that directly handle HTTP requests.
{
public:
	virtual void handleForm(const std::string& field, const std::string& value) = 0;
		/// Handles a form field submitted by the client.
	
	virtual void handleAjaxRequest(const Poco::Net::NameValueCollection& args, Poco::Net::HTTPServerResponse& response) = 0;
		/// Handles a complete AJAX request submitted by the client.

	virtual bool serializeJSON(std::ostream& out, const std::string& name) = 0;
		/// Serializes a form field to the client.
protected:
	RequestProcessor();
		/// Creates the RequestProcessor.
		
	virtual ~RequestProcessor();
		/// Destroys the RequestProcessor.
};


} } // namespace Poco::WebWidgets


#endif // WebWidgets_RequestProcessor_INCLUDED
